"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cms = void 0;
const fs_1 = require("fs");
const crypto = require("crypto");
const asn1js = require("asn1js");
const pkijs_1 = require("pkijs");
const { subtle } = crypto.webcrypto;
const engine = new pkijs_1.CryptoEngine({ subtle, crypto });
(0, pkijs_1.setEngine)("node", engine, engine);
function load(file) {
    const contents = (0, fs_1.readFileSync)(file, "utf-8");
    return Buffer.from(contents
        .replace(/[-]+.+[-]+/g, "")
        .trim(), "base64");
}
function toArrayBuffer(buf) {
    return buf.buffer.slice(buf.byteOffset, buf.byteOffset + buf.length);
}
async function cms(cert, pk, content) {
    const buffer = toArrayBuffer(Buffer.from(content, "utf-8"));
    const certBuf = toArrayBuffer(load(cert));
    const keyBuf = toArrayBuffer(load(pk));
    const x509 = asn1js.fromBER(certBuf);
    const certificate = new pkijs_1.Certificate({ schema: x509.result });
    const digest = await engine.digest({ name: "sha-256" }, buffer);
    const cmsSigned = new pkijs_1.SignedData({
        version: 1,
        digestAlgorithms: [
            new pkijs_1.AlgorithmIdentifier({ algorithmId: "2.16.840.1.101.3.4.2.1" }),
        ],
        encapContentInfo: new pkijs_1.EncapsulatedContentInfo({
            eContentType: "1.2.840.113549.1.7.1",
        }),
        signerInfos: [
            new pkijs_1.SignerInfo({
                version: 1,
                sid: new pkijs_1.IssuerAndSerialNumber({
                    issuer: certificate.issuer,
                    serialNumber: certificate.serialNumber,
                }),
                signedAttrs: new pkijs_1.SignedAndUnsignedAttributes({
                    type: 0,
                    attributes: [
                        new pkijs_1.Attribute({
                            type: "1.2.840.113549.1.9.3",
                            values: [
                                new asn1js.ObjectIdentifier({ value: "1.2.840.113549.1.7.1" }),
                            ],
                        }),
                        new pkijs_1.Attribute({
                            type: "1.2.840.113549.1.9.5",
                            values: [
                                new asn1js.UTCTime({ valueDate: new Date() }),
                            ],
                        }),
                        new pkijs_1.Attribute({
                            type: "1.2.840.113549.1.9.4",
                            values: [
                                new asn1js.OctetString({ valueHex: digest }),
                            ],
                        }),
                    ],
                }),
            }),
        ],
        certificates: [certificate],
    });
    const cryptoKey = await engine.importKey("pkcs8", keyBuf, { name: "ECDSA", namedCurve: "P-384" }, true, ["sign"]);
    // detached mode
    await cmsSigned.sign(cryptoKey, 0, "sha-256", buffer);
    const cmsContentSimp = new pkijs_1.ContentInfo({
        contentType: "1.2.840.113549.1.7.2",
        content: cmsSigned.toSchema(true),
    });
    const cmsContentSchema = cmsContentSimp.toSchema();
    const binary = cmsContentSchema.toBER(false);
    return Buffer.from(binary).toString("base64");
}
exports.cms = cms;
//# sourceMappingURL=cms.js.map